@@ -41,19 +41,22 @@
 
 #include "nsIAccessibleStates.h"
 #include "nsIAccessibleTypes.h"
-#include "nsPIAccessible.h"
 
 #include "nsAccessibleEventData.h"
 #include "nsHyperTextAccessible.h"
+#include "nsHTMLTableAccessible.h"
+#include "nsDocAccessible.h"
 #include "nsAccessibilityAtoms.h"
 #include "nsAccessibleTreeWalker.h"
 #include "nsAccessible.h"
 #include "nsARIAMap.h"
+#include "nsXULTreeGridAccessible.h"
 
 #include "nsIDOMXULContainerElement.h"
 #include "nsIDOMXULSelectCntrlEl.h"
 #include "nsIDOMXULSelectCntrlItemEl.h"
 #include "nsWhitespaceTokenizer.h"
+#include "nsComponentManagerUtils.h"
 
 void
 nsAccUtils::GetAccAttr(nsIPersistentProperties *aAttributes,
@@ -272,21 +275,41 @@ nsAccUtils::SetLiveContainerAttributes(nsIPersistentProperties *aAttributes,
   nsAutoString atomic, live, relevant, busy;
   nsIContent *ancestor = aStartContent;
   while (ancestor) {
+
+    // container-relevant attribute
     if (relevant.IsEmpty() &&
         nsAccUtils::HasDefinedARIAToken(ancestor, nsAccessibilityAtoms::aria_relevant) &&
         ancestor->GetAttr(kNameSpaceID_None, nsAccessibilityAtoms::aria_relevant, relevant))
       SetAccAttr(aAttributes, nsAccessibilityAtoms::containerRelevant, relevant);
 
-    if (live.IsEmpty() &&
-        nsAccUtils::HasDefinedARIAToken(ancestor, nsAccessibilityAtoms::aria_live) &&
-        ancestor->GetAttr(kNameSpaceID_None, nsAccessibilityAtoms::aria_live, live))
-      SetAccAttr(aAttributes, nsAccessibilityAtoms::containerLive, live);
+    // container-live, and container-live-role attributes
+    if (live.IsEmpty()) {
+      nsCOMPtr<nsIDOMNode> node(do_QueryInterface(ancestor));
+      nsRoleMapEntry *role = GetRoleMapEntry(node);
+      if (nsAccUtils::HasDefinedARIAToken(ancestor,
+                                          nsAccessibilityAtoms::aria_live)) {
+        ancestor->GetAttr(kNameSpaceID_None, nsAccessibilityAtoms::aria_live,
+                          live);
+      } else if (role) {
+        GetLiveAttrValue(role->liveAttRule, live);
+      }
+      if (!live.IsEmpty()) {
+        SetAccAttr(aAttributes, nsAccessibilityAtoms::containerLive, live);
+        if (role) {
+          nsAccUtils::SetAccAttr(aAttributes,
+                                 nsAccessibilityAtoms::containerLiveRole,
+                                 NS_ConvertASCIItoUTF16(role->roleString));
+        }
+      }
+    }
 
+    // container-atomic attribute
     if (atomic.IsEmpty() &&
         nsAccUtils::HasDefinedARIAToken(ancestor, nsAccessibilityAtoms::aria_atomic) &&
         ancestor->GetAttr(kNameSpaceID_None, nsAccessibilityAtoms::aria_atomic, atomic))
       SetAccAttr(aAttributes, nsAccessibilityAtoms::containerAtomic, atomic);
 
+    // container-busy attribute
     if (busy.IsEmpty() &&
         nsAccUtils::HasDefinedARIAToken(ancestor, nsAccessibilityAtoms::aria_busy) &&
         ancestor->GetAttr(kNameSpaceID_None, nsAccessibilityAtoms::aria_busy, busy))
@@ -304,6 +327,8 @@ nsAccUtils::SetLiveContainerAttributes(nsIPersistentProperties *aAttributes,
 PRBool
 nsAccUtils::HasDefinedARIAToken(nsIContent *aContent, nsIAtom *aAtom)
 {
+  NS_ASSERTION(aContent, "aContent is null in call to HasDefinedARIAToken!");
+
   if (!aContent->HasAttr(kNameSpaceID_None, aAtom) ||
       aContent->AttrValueIs(kNameSpaceID_None, aAtom,
                             nsAccessibilityAtoms::_empty, eCaseMatters) ||
@@ -320,14 +345,13 @@ nsAccUtils::FireAccEvent(PRUint32 aEventType, nsIAccessible *aAccessible,
 {
   NS_ENSURE_ARG(aAccessible);
 
-  nsCOMPtr<nsPIAccessible> pAccessible(do_QueryInterface(aAccessible));
-  NS_ASSERTION(pAccessible, "Accessible doesn't implement nsPIAccessible");
+  nsRefPtr<nsAccessible> acc(nsAccUtils::QueryAccessible(aAccessible));
 
   nsCOMPtr<nsIAccessibleEvent> event =
     new nsAccEvent(aEventType, aAccessible, aIsAsynch);
   NS_ENSURE_TRUE(event, NS_ERROR_OUT_OF_MEMORY);
 
-  return pAccessible->FireAccessibleEvent(event);
+  return acc->FireAccessibleEvent(event);
 }
 
 PRBool
@@ -363,8 +387,7 @@ nsAccUtils::GetAncestorWithRole(nsIAccessible *aDescendant, PRUint32 aRole)
   nsCOMPtr<nsIAccessible> parentAccessible = aDescendant, testRoleAccessible;
   while (NS_SUCCEEDED(parentAccessible->GetParent(getter_AddRefs(testRoleAccessible))) &&
          testRoleAccessible) {
-    PRUint32 testRole;
-    testRoleAccessible->GetFinalRole(&testRole);
+    PRUint32 testRole = nsAccUtils::Role(testRoleAccessible);
     if (testRole == aRole) {
       nsIAccessible *returnAccessible = testRoleAccessible;
       NS_ADDREF(returnAccessible);
@@ -385,14 +408,30 @@ nsAccUtils::GetARIATreeItemParent(nsIAccessible *aStartTreeItem,
                                   nsIAccessible **aTreeItemParentResult)
 {
   *aTreeItemParentResult = nsnull;
+
+  nsCOMPtr<nsIAccessible> parentAccessible;
+  aStartTreeItem->GetParent(getter_AddRefs(parentAccessible));
+  if (!parentAccessible)
+    return;
+
+  PRUint32 startTreeItemRole = nsAccUtils::Role(aStartTreeItem);
+
+  // Calculate tree grid row parent only if the row inside of ARIA treegrid.
+  if (startTreeItemRole == nsIAccessibleRole::ROLE_ROW) {
+    PRUint32 role = nsAccUtils::Role(parentAccessible);
+    if (role != nsIAccessibleRole::ROLE_TREE_TABLE)
+      return;
+  }
+
+  // This is a tree or treegrid that uses aria-level to define levels, so find
+  // the first previous sibling accessible where level is defined to be less
+  // than the current level.
   nsAutoString levelStr;
-  PRInt32 level = 0;
   if (nsAccUtils::HasDefinedARIAToken(aStartContent, nsAccessibilityAtoms::aria_level) &&
       aStartContent->GetAttr(kNameSpaceID_None, nsAccessibilityAtoms::aria_level, levelStr)) {
-    // This is a tree that uses aria-level to define levels, so find the first previous
-    // sibling accessible where level is defined to be less than the current level
+
     PRInt32 success;
-    level = levelStr.ToInteger(&success);
+    PRInt32 level = levelStr.ToInteger(&success);
     if (level > 1 && NS_SUCCEEDED(success)) {
       nsCOMPtr<nsIAccessible> currentAccessible = aStartTreeItem, prevAccessible;
       while (PR_TRUE) {
@@ -402,10 +441,10 @@ nsAccUtils::GetARIATreeItemParent(nsIAccessible *aStartTreeItem,
         if (!accessNode) {
           break; // Reached top of tree, no higher level found
         }
-        PRUint32 role;
-        currentAccessible->GetFinalRole(&role);
-        if (role != nsIAccessibleRole::ROLE_OUTLINEITEM)
+        PRUint32 role = nsAccUtils::Role(currentAccessible);
+        if (role != startTreeItemRole)
           continue;
+
         nsCOMPtr<nsIDOMNode> treeItemNode;
         accessNode->GetDOMNode(getter_AddRefs(treeItemNode));
         nsCOMPtr<nsIContent> treeItemContent = do_QueryInterface(treeItemNode);
@@ -423,25 +462,30 @@ nsAccUtils::GetARIATreeItemParent(nsIAccessible *aStartTreeItem,
     }
   }
 
-  // Possibly a tree arranged by using role="group" to organize levels
-  // In this case the parent of the tree item will be a group and the
-  // previous sibling of that should be the tree item parent.
-  // Or, if the parent is something other than a tree we will return that.
-  nsCOMPtr<nsIAccessible> parentAccessible;
-  aStartTreeItem->GetParent(getter_AddRefs(parentAccessible));
-  if (!parentAccessible)
-    return;
-  PRUint32 role;
-  parentAccessible->GetFinalRole(&role);
+  // In the case of ARIA treegrid, return its parent since ARIA group isn't
+  // used to organize levels in ARIA treegrids.
+
+  if (startTreeItemRole == nsIAccessibleRole::ROLE_ROW) {
+    NS_ADDREF(*aTreeItemParentResult = parentAccessible);
+    return; // The container for the tree grid rows
+  }
+
+  // In the case of ARIA tree, a tree can be arranged by using role="group" to
+  // organize levels. In this case the parent of the tree item will be a group
+  // and the previous sibling of that should be the tree item parent. Or, if
+  // the parent is something other than a tree we will return that.
+
+  PRUint32 role = nsAccUtils::Role(parentAccessible);
   if (role != nsIAccessibleRole::ROLE_GROUPING) {
     NS_ADDREF(*aTreeItemParentResult = parentAccessible);
     return; // The container for the tree items
   }
+
   nsCOMPtr<nsIAccessible> prevAccessible;
   parentAccessible->GetPreviousSibling(getter_AddRefs(prevAccessible));
   if (!prevAccessible)
     return;
-  prevAccessible->GetFinalRole(&role);
+  role = nsAccUtils::Role(prevAccessible);
   if (role == nsIAccessibleRole::ROLE_TEXT_LEAF) {
     // XXX Sometimes an empty text accessible is in the hierarchy here,
     // although the text does not appear to be rendered, GetRenderedText() says that it is
@@ -450,7 +494,7 @@ nsAccUtils::GetARIATreeItemParent(nsIAccessible *aStartTreeItem,
     tempAccessible->GetPreviousSibling(getter_AddRefs(prevAccessible));
     if (!prevAccessible)
       return;
-    prevAccessible->GetFinalRole(&role);
+    role = nsAccUtils::Role(prevAccessible);
   }
   if (role == nsIAccessibleRole::ROLE_OUTLINEITEM) {
     // Previous sibling of parent group is a tree item -- this is the conceptual tree item parent
@@ -458,6 +502,25 @@ nsAccUtils::GetARIATreeItemParent(nsIAccessible *aStartTreeItem,
   }
 }
 
+PRBool
+nsAccUtils::IsARIASelected(nsIAccessible *aAccessible)
+{
+  nsRefPtr<nsAccessible> acc = nsAccUtils::QueryAccessible(aAccessible);
+  nsCOMPtr<nsIDOMNode> node;
+  acc->GetDOMNode(getter_AddRefs(node));
+  NS_ASSERTION(node, "No DOM node!");
+
+  if (node) {
+    nsCOMPtr<nsIContent> content(do_QueryInterface(node));
+    if (content->AttrValueIs(kNameSpaceID_None,
+                             nsAccessibilityAtoms::aria_selected,
+                             nsAccessibilityAtoms::_true, eCaseMatters))
+      return PR_TRUE;
+  }
+
+  return PR_FALSE;
+}
+
 already_AddRefed<nsIAccessibleText>
 nsAccUtils::GetTextAccessibleFromSelection(nsISelection *aSelection,
                                            nsIDOMNode **aNode)
@@ -465,29 +528,16 @@ nsAccUtils::GetTextAccessibleFromSelection(nsISelection *aSelection,
   // Get accessible from selection's focus DOM point (the DOM point where
   // selection is ended).
 
-  nsCOMPtr<nsIDOMNode> resultNode;
-  aSelection->GetFocusNode(getter_AddRefs(resultNode));
-  if (!resultNode)
+  nsCOMPtr<nsIDOMNode> focusNode;
+  aSelection->GetFocusNode(getter_AddRefs(focusNode));
+  if (!focusNode)
     return nsnull;
 
-  // Get DOM node that focus DOM point points to.
-  nsCOMPtr<nsIContent> content(do_QueryInterface(resultNode));
-  if (content && content->IsNodeOfType(nsINode::eELEMENT)) {
-    PRInt32 offset = 0;
-    aSelection->GetFocusOffset(&offset);
+  PRInt32 focusOffset = 0;
+  aSelection->GetFocusOffset(&focusOffset);
 
-    PRInt32 childCount = static_cast<PRInt32>(content->GetChildCount());
-    NS_ASSERTION(offset >= 0 && offset <= childCount,
-                 "Wrong focus offset in selection!");
-
-    // The offset can be after last child of container node that means DOM point
-    // is placed immediately after the last child. In this case use focusNode
-    // as result node.
-    if (offset != childCount) {
-      nsCOMPtr<nsIContent> child = content->GetChildAt(offset);
-      resultNode = do_QueryInterface(child);
-    }
-  }
+  nsCOMPtr<nsIDOMNode> resultNode =
+    nsCoreUtils::GetDOMNodeFromDOMPoint(focusNode, focusOffset);
 
   nsIAccessibilityService *accService = nsAccessNode::GetAccService();
 
@@ -648,18 +698,18 @@ nsAccUtils::GetRoleMapEntry(nsIDOMNode *aNode)
   nsWhitespaceTokenizer tokenizer(roleString);
   while (tokenizer.hasMoreTokens()) {
     // Do a binary search through table for the next role in role list
-    const char *role = NS_LossyConvertUTF16toASCII(tokenizer.nextToken()).get();
-    PRInt32 low = 0;
-    PRInt32 high = nsARIAMap::gWAIRoleMapLength;
-    while (low <= high) {
-      PRInt32 index = low + ((high - low) / 2);
-      PRInt32 compare = PL_strcmp(role, nsARIAMap::gWAIRoleMap[index].roleString);
+    NS_LossyConvertUTF16toASCII role(tokenizer.nextToken());
+    PRUint32 low = 0;
+    PRUint32 high = nsARIAMap::gWAIRoleMapLength;
+    while (low < high) {
+      PRUint32 index = (low + high) / 2;
+      PRInt32 compare = PL_strcmp(role.get(), nsARIAMap::gWAIRoleMap[index].roleString);
       if (compare == 0) {
         // The  role attribute maps to an entry in the role table
         return &nsARIAMap::gWAIRoleMap[index];
       }
       if (compare < 0) {
-        high = index - 1;
+        high = index;
       }
       else {
         low = index + 1;
@@ -672,6 +722,23 @@ nsAccUtils::GetRoleMapEntry(nsIDOMNode *aNode)
   return &nsARIAMap::gLandmarkRoleMap;
 }
 
+PRUint32
+nsAccUtils::RoleInternal(nsIAccessible *aAcc)
+{
+  PRUint32 role = nsIAccessibleRole::ROLE_NOTHING;
+  if (aAcc) {
+    nsAccessible* accessible = nsnull;
+    CallQueryInterface(aAcc, &accessible);
+
+    if (accessible) {
+      accessible->GetRoleInternal(&role);
+      NS_RELEASE(accessible);
+    }
+  }
+
+  return role;
+}
+
 PRUint8
 nsAccUtils::GetAttributeCharacteristics(nsIAtom* aAtom)
 {
@@ -682,6 +749,82 @@ nsAccUtils::GetAttributeCharacteristics(nsIAtom* aAtom)
     return 0;
 }
 
+PRBool
+nsAccUtils::GetLiveAttrValue(PRUint32 aRule, nsAString& aValue)
+{
+  switch (aRule) {
+    case eOffLiveAttr:
+      aValue = NS_LITERAL_STRING("off");
+      return PR_TRUE;
+    case ePoliteLiveAttr:
+      aValue = NS_LITERAL_STRING("polite");
+      return PR_TRUE;
+  }
+
+  return PR_FALSE;
+}
+
+already_AddRefed<nsAccessible>
+nsAccUtils::QueryAccessible(nsIAccessible *aAccessible)
+{
+  nsAccessible* acc = nsnull;
+  if (aAccessible)
+    CallQueryInterface(aAccessible, &acc);
+
+  return acc;
+}
+
+already_AddRefed<nsAccessible>
+nsAccUtils::QueryAccessible(nsIAccessNode *aAccessNode)
+{
+  nsAccessible* acc = nsnull;
+  if (aAccessNode)
+    CallQueryInterface(aAccessNode, &acc);
+
+  return acc;
+}
+
+already_AddRefed<nsHTMLTableAccessible>
+nsAccUtils::QueryAccessibleTable(nsIAccessibleTable *aAccessibleTable)
+{
+  nsHTMLTableAccessible* accessible = nsnull;
+  if (aAccessibleTable)
+    CallQueryInterface(aAccessibleTable, &accessible);
+  
+  return accessible;
+}
+
+already_AddRefed<nsDocAccessible>
+nsAccUtils::QueryAccessibleDocument(nsIAccessible *aAccessible)
+{
+  nsDocAccessible* accessible = nsnull;
+  if (aAccessible)
+    CallQueryInterface(aAccessible, &accessible);
+
+  return accessible;
+}
+
+already_AddRefed<nsDocAccessible>
+nsAccUtils::QueryAccessibleDocument(nsIAccessibleDocument *aAccessibleDocument)
+{
+  nsDocAccessible* accessible = nsnull;
+  if (aAccessibleDocument)
+    CallQueryInterface(aAccessibleDocument, &accessible);
+
+  return accessible;
+}
+
+#ifdef MOZ_XUL
+already_AddRefed<nsXULTreeAccessible>
+nsAccUtils::QueryAccessibleTree(nsIAccessible *aAccessible)
+{
+  nsXULTreeAccessible* accessible = nsnull;
+  if (aAccessible)
+    CallQueryInterface(aAccessible, &accessible);
+
+  return accessible;
+}
+#endif
 
 #ifdef DEBUG_A11Y
 
@@ -744,10 +887,10 @@ nsAccUtils::TextLength(nsIAccessible *aAccessible)
   // text. They don't have their own frame.
   // XXX In the future, list bullets may have frame and anon content, so 
   // we should be able to remove this at that point
-  nsCOMPtr<nsPIAccessible> pAcc(do_QueryInterface(aAccessible));
-  
+  nsRefPtr<nsAccessible> acc(nsAccUtils::QueryAccessible(aAccessible));
+
   nsAutoString text;
-  pAcc->AppendTextTo(text, 0, PR_UINT32_MAX); // Get all the text
+  acc->AppendTextTo(text, 0, PR_UINT32_MAX); // Get all the text
   return text.Length();
 }
 
@@ -795,12 +938,11 @@ nsAccUtils::GetMultiSelectFor(nsIDOMNode *aNode)
   if (0 == (state & nsIAccessibleStates::STATE_SELECTABLE))
     return nsnull;
 
-  PRUint32 containerRole;
   while (0 == (state & nsIAccessibleStates::STATE_MULTISELECTABLE)) {
     nsIAccessible *current = accessible;
     current->GetParent(getter_AddRefs(accessible));
-    if (!accessible || (NS_SUCCEEDED(accessible->GetFinalRole(&containerRole)) &&
-                        containerRole == nsIAccessibleRole::ROLE_PANE)) {
+    if (!accessible ||
+        nsAccUtils::Role(accessible) == nsIAccessibleRole::ROLE_PANE) {
       return nsnull;
     }
     state = State(accessible);
@@ -810,3 +952,58 @@ nsAccUtils::GetMultiSelectFor(nsIDOMNode *aNode)
   accessible.swap(returnAccessible);
   return returnAccessible;
 }
+
+nsresult
+nsAccUtils::GetHeaderCellsFor(nsIAccessibleTable *aTable,
+                              nsIAccessibleTableCell *aCell,
+                              PRInt32 aRowOrColHeaderCells, nsIArray **aCells)
+{
+  nsresult rv = NS_OK;
+  nsCOMPtr<nsIMutableArray> cells = do_CreateInstance(NS_ARRAY_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  PRInt32 rowIdx = -1;
+  rv = aCell->GetRowIndex(&rowIdx);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  PRInt32 colIdx = -1;
+  rv = aCell->GetColumnIndex(&colIdx);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  PRBool moveToLeft = aRowOrColHeaderCells == eRowHeaderCells;
+
+  // Move to the left or top to find row header cells or column header cells.
+  PRInt32 index = (moveToLeft ? colIdx : rowIdx) - 1;
+  for (; index >= 0; index--) {
+    PRInt32 curRowIdx = moveToLeft ? rowIdx : index;
+    PRInt32 curColIdx = moveToLeft ? index : colIdx;
+
+    nsCOMPtr<nsIAccessible> cell;
+    rv = aTable->GetCellAt(curRowIdx, curColIdx, getter_AddRefs(cell));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    nsCOMPtr<nsIAccessibleTableCell> tableCellAcc =
+      do_QueryInterface(cell);
+
+    PRInt32 origIdx = 1;
+    if (moveToLeft)
+      rv = tableCellAcc->GetColumnIndex(&origIdx);
+    else
+      rv = tableCellAcc->GetRowIndex(&origIdx);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    if (origIdx == index) {
+      // Append original header cells only.
+      PRUint32 role = Role(cell);
+      PRBool isHeader = moveToLeft ?
+        role == nsIAccessibleRole::ROLE_ROWHEADER :
+        role == nsIAccessibleRole::ROLE_COLUMNHEADER;
+
+      if (isHeader)
+        cells->AppendElement(cell, PR_FALSE);
+    }
+  }
+
+  NS_ADDREF(*aCells = cells);
+  return NS_OK;
+}
