@@ -36,49 +36,28 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '15.8-1.js';
+gTestfile = '15.4.3.js';
 
 /**
-   File Name:          15.8-1.js
-   ECMA Section:       15.8 The Math Object
-
-   Description:
-
-   The Math object is merely a single object that has some named properties,
-   some of which are functions.
-
-   The value of the internal [[Prototype]] property of the Math object is the
-   Object prototype object (15.2.3.1).
-
-   The Math object does not have a [[Construct]] property; it is not possible
-   to use the Math object as a constructor with the new operator.
-
-   The Math object does not have a [[Call]] property; it is not possible to
-   invoke the Math object as a function.
-
-   Recall that, in this specification, the phrase "the number value for x" has
-   a technical meaning defined in section 8.5.
+   File Name:          15.4.3.js
+   ECMA Section:       15.4.3 Properties of the Array Constructor
+   Description:        The value of the internal [[Prototype]] property of the
+   Array constructor is the Function prototype object.
 
    Author:             christine@netscape.com
-   Date:               12 november 1997
-
+   Date:               7 october 1997
 */
 
-var SECTION = "15.8-1";
+var SECTION = "15.4.3";
 var VERSION = "ECMA_2";
 startTest();
-var TITLE   = "The Math Object";
+var TITLE   = "Properties of the Array Constructor";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
 new TestCase( SECTION,
-	      "Math.__proto__ == Object.prototype",
-	      true,
-	      Math.__proto__ == Object.prototype );
-
-new TestCase( SECTION,
-	      "Math.__proto__",
-	      Object.prototype,
-	      Math.__proto__ );
+	      "Array.__proto__",     
+	      Function.prototype,       
+	      Array.__proto__ );
 
 test();
