@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2006
+ * Portions created by the Initial Developer are Copyright (C) 2007
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jesse Ruderman
+ * Contributor(s): Gary Kwong
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,10 +35,10 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-349482-02.js';
+var gTestfile = 'regress-387951-03.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 349482;
-var summary = 'Decompiling try/catch in with() should not crash';
+var BUGNUMBER = 387951;
+var summary = 'Do not assert: cg->stackDepth >= 0';
 var actual = 'No Crash';
 var expect = 'No Crash';
 
@@ -53,8 +53,7 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
  
-  var f = function() { with({})     { try{}catch(e){} } }
-  print(f.toString());
+  switch(delete[null?0:{}]){default:}
 
   reportCompare(expect, actual, summary);
 
