@@ -16,7 +16,7 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2006
+ * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s): Igor Bukanov
@@ -35,28 +35,32 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-352885-01.js';
+var gTestfile = 'regress-462734-04.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 352885;
-var summary = 'Do not crash iterating over gen.__proto__';
-var actual = 'No Crash';
-var expect = 'No Crash';
+var BUGNUMBER = 462734;
+var summary = 'Do not assert: pobj_ == obj2';
+var actual = '';
+var expect = '';
 
+printBugNumber(BUGNUMBER);
+printStatus (summary);
 
-//-----------------------------------------------------------------------------
-test();
-//-----------------------------------------------------------------------------
+var save__proto__ = __proto__;
 
-function test()
+try
 {
-  enterFunc ('test');
-  printBugNumber(BUGNUMBER);
-  printStatus (summary);
- 
-  for (j in ((function() { yield 3; })().__proto__))
-    print(j);
-
-  reportCompare(expect, actual, summary);
-
-  exitFunc ('test');
+  Function.prototype.prototype = function() { return 42; }
+  __proto__ = Function();
+  prototype = prototype;
 }
+catch(ex)
+{
+  print(ex + '');
+}
+
+expect = 'object';
+actual = typeof prototype;
+
+__proto__ = save__proto__;
+
+reportCompare(expect, actual, summary);
