@@ -47,6 +47,8 @@
 #include "nsAutoPtr.h"
 #include "nsStyleSet.h"
 #include "nsFrameManager.h"
+#include "nsPlaceholderFrame.h"
+#include "nsCSSFrameConstructor.h"
 
 nsIFrame*
 NS_NewFirstLetterFrame(nsIPresShell* aPresShell, nsStyleContext* aContext)
@@ -56,6 +58,10 @@ NS_NewFirstLetterFrame(nsIPresShell* aPresShell, nsStyleContext* aContext)
 
 NS_IMPL_FRAMEARENA_HELPERS(nsFirstLetterFrame)
 
+NS_QUERYFRAME_HEAD(nsFirstLetterFrame)
+  NS_QUERYFRAME_ENTRY(nsFirstLetterFrame)
+NS_QUERYFRAME_TAIL_INHERITING(nsFirstLetterFrameSuper)
+
 #ifdef NS_DEBUG
 NS_IMETHODIMP
 nsFirstLetterFrame::GetFrameName(nsAString& aResult) const
@@ -267,23 +273,32 @@ nsFirstLetterFrame::Reflow(nsPresContext*          aPresContext,
   else {
     // Create a continuation for the child frame if it doesn't already
     // have one.
-    nsIFrame* nextInFlow;
-    rv = CreateNextInFlow(aPresContext, this, kid, nextInFlow);
-    if (NS_FAILED(rv)) {
-      return rv;
-    }
-
-    // And then push it to our overflow list
-    if (nextInFlow) {
-      kid->SetNextSibling(nsnull);
-      SetOverflowFrames(aPresContext, nextInFlow);
-    }
-    else {
-      nsIFrame* nextSib = kid->GetNextSibling();
-      if (nextSib) {
-        kid->SetNextSibling(nsnull);
-        SetOverflowFrames(aPresContext, nextSib);
+    if (!GetStyleDisplay()->IsFloating()) {
+      nsIFrame* nextInFlow;
+      rv = CreateNextInFlow(aPresContext, this, kid, nextInFlow);
+      if (NS_FAILED(rv)) {
+        return rv;
       }
+
+      // And then push it to our overflow list
+      if (nextInFlow) {
+        kid->SetNextSibling(nsnull);
+        SetOverflowFrames(aPresContext, nextInFlow);
+      }
+      else {
+        nsIFrame* nextSib = kid->GetNextSibling();
+        if (nextSib) {
+          kid->SetNextSibling(nsnull);
+          SetOverflowFrames(aPresContext, nextSib);
+        }
+      }
+    } else if (!kid->GetNextInFlow()) {
+      // For floating first letter frames (if a continuation wasn't already
+      // created for us) we need to put the continuation with the rest of the
+      // text that the first letter frame was made out of.
+      nsIFrame* continuation;
+      rv = CreateContinuationForFloatingParent(aPresContext, kid,
+                                               &continuation, PR_TRUE);
     }
   }
 
@@ -300,6 +315,54 @@ nsFirstLetterFrame::CanContinueTextRun() const
   return PR_TRUE;
 }
 
+nsresult
+nsFirstLetterFrame::CreateContinuationForFloatingParent(nsPresContext* aPresContext,
+                                                        nsIFrame* aChild,
+                                                        nsIFrame** aContinuation,
+                                                        PRBool aIsFluid)
+{
+  NS_ASSERTION(GetStyleDisplay()->IsFloating(),
+               "can only call this on floating first letter frames");
+  NS_PRECONDITION(aContinuation, "bad args");
+
+  *aContinuation = nsnull;
+  nsresult rv = NS_OK;
+
+  nsIPresShell* presShell = aPresContext->PresShell();
+  nsPlaceholderFrame* placeholderFrame =
+    presShell->FrameManager()->GetPlaceholderFrameFor(this);
+  nsIFrame* parent = placeholderFrame->GetParent();
+
+  nsIFrame* continuation;
+  rv = presShell->FrameConstructor()->
+    CreateContinuingFrame(aPresContext, aChild, parent, &continuation, aIsFluid);
+  if (NS_FAILED(rv) || !continuation) {
+    return rv;
+  }
+
+  // The continuation will have gotten the first letter style from it's
+  // prev continuation, so we need to repair the style context so it
+  // doesn't have the first letter styling.
+  nsStyleContext* parentSC = this->GetStyleContext()->GetParent();
+  if (parentSC) {
+    nsRefPtr<nsStyleContext> newSC;
+    newSC = presShell->StyleSet()->ResolveStyleForNonElement(parentSC);
+    if (newSC) {
+      continuation->SetStyleContext(newSC);
+    }
+  }
+
+  //XXX Bidi may not be involved but we have to use the list name
+  // nsGkAtoms::nextBidi because this is just like creating a continuation
+  // except we have to insert it in a different place and we don't want a
+  // reflow command to try to be issued.
+  nsFrameList temp(continuation);
+  rv = parent->InsertFrames(nsGkAtoms::nextBidi, placeholderFrame, temp);
+
+  *aContinuation = continuation;
+  return rv;
+}
+
 void
 nsFirstLetterFrame::DrainOverflowFrames(nsPresContext* aPresContext)
 {
