@@ -67,6 +67,8 @@
 #include "nsString.h"
 #include "nsXPIDLString.h"
 #include "plstr.h" // PL_strcasestr(...)
+#include "nsNetUtil.h"
+#include "nsIProtocolHandler.h"
 
 #if defined(PR_LOGGING)
 PRLogModuleInfo *gImgLog = PR_NewLogModule("imgRequest");
@@ -709,7 +711,14 @@ NS_IMETHODIMP imgRequest::OnStartRequest(nsIRequest *aRequest, nsISupports *ctxt
 
   /* set our state variables to their initial values, but advance mState
      to onStartRequest. */
-  mImageStatus = imgIRequest::STATUS_NONE;
+  if (mIsMultiPartChannel) {
+    // Don't blow away our status altogether
+    mImageStatus &= ~imgIRequest::STATUS_LOAD_PARTIAL;
+    mImageStatus &= ~imgIRequest::STATUS_LOAD_COMPLETE;
+    mImageStatus &= ~imgIRequest::STATUS_FRAME_COMPLETE;
+  } else {
+    mImageStatus = imgIRequest::STATUS_NONE;
+  }
   mState = onStartRequest;
 
   nsCOMPtr<nsIChannel> channel(do_QueryInterface(aRequest));
@@ -1077,6 +1086,18 @@ imgRequest::OnChannelRedirect(nsIChannel *oldChannel, nsIChannel *newChannel, PR
     mKeyURI->GetSpec(oldspec);
   LOG_MSG_WITH_PARAM(gImgLog, "imgRequest::OnChannelRedirect", "old", oldspec.get());
 
+  // make sure we have a protocol that returns data rather than opens
+  // an external application, e.g. mailto:
+  nsCOMPtr<nsIURI> uri;
+  newChannel->GetURI(getter_AddRefs(uri));
+  PRBool doesNotReturnData = PR_FALSE;
+  rv = NS_URIChainHasFlags(uri, nsIProtocolHandler::URI_DOES_NOT_RETURN_DATA,
+                           &doesNotReturnData);
+  if (NS_FAILED(rv))
+    return rv;
+  if (doesNotReturnData)
+    return NS_ERROR_ABORT;
+
   nsCOMPtr<nsIURI> newURI;
   newChannel->GetOriginalURI(getter_AddRefs(newURI));
   nsCAutoString newspec;
